@@ -1,4 +1,4 @@
-use argmin::core::Error as ArgminError;
+use crate::parameter::ParameterError;
 use num_dual::linalg::LinAlgError;
 use quantity::QuantityError;
 use thiserror::Error;
@@ -19,13 +19,15 @@ pub enum EosError {
     #[error("Undetermined state: {0}.")]
     UndeterminedState(String),
     #[error("System is supercritical.")]
-    SuperCritical(),
+    SuperCritical,
     #[error("No phase split according to stability analysis.")]
     NoPhaseSplit,
+    #[error("Wrong input units. Expected {0}, got {1}")]
+    WrongUnits(String, String),
     #[error(transparent)]
     QuantityError(#[from] QuantityError),
     #[error(transparent)]
-    ArgminError(#[from] ArgminError),
+    ParameterError(#[from] ParameterError),
     #[error(transparent)]
     LinAlgError(#[from] LinAlgError),
 }
