@@ -1,4 +1,4 @@
-use super::{PhaseEquilibrium, VLEOptions, Verbosity};
+use super::{PhaseEquilibrium, SolverOptions, Verbosity};
 use crate::equation_of_state::EquationOfState;
 use crate::errors::{EosError, EosResult};
 use crate::state::{Contributions, DensityInitialization, State};
@@ -23,8 +23,8 @@ impl<U: EosUnit, E: EquationOfState> PhaseEquilibrium<U, E, 2> {
         temperature: QuantityScalar<U>,
         pressure: QuantityScalar<U>,
         feed: &QuantityArray1<U>,
-        init_vle_state: Option<&PhaseEquilibrium<U, E, 2>>,
-        options: VLEOptions,
+        initial_state: Option<&PhaseEquilibrium<U, E, 2>>,
+        options: SolverOptions,
         non_volatile_components: Option<Vec<usize>>,
     ) -> EosResult<Self> {
         State::new_npt(
@@ -34,7 +34,7 @@ impl<U: EosUnit, E: EquationOfState> PhaseEquilibrium<U, E, 2> {
             feed,
             DensityInitialization::None,
         )?
-        .tp_flash(init_vle_state, options, non_volatile_components)
+        .tp_flash(initial_state, options, non_volatile_components)
     }
 }
 
@@ -48,15 +48,15 @@ impl<U: EosUnit, E: EquationOfState> State<U, E> {
     /// containing non-volatile components (e.g. ions).
     pub fn tp_flash(
         &self,
-        init_vle_state: Option<&PhaseEquilibrium<U, E, 2>>,
-        options: VLEOptions,
+        initial_state: Option<&PhaseEquilibrium<U, E, 2>>,
+        options: SolverOptions,
         non_volatile_components: Option<Vec<usize>>,
     ) -> EosResult<PhaseEquilibrium<U, E, 2>> {
         // set options
         let (max_iter, tol, verbosity) = options.unwrap_or(MAX_ITER_TP, TOL_TP);
 
         // initialization
-        let mut new_vle_state = match init_vle_state {
+        let mut new_vle_state = match initial_state {
             Some(init) => init
                 .clone()
                 .update_pressure(self.temperature, self.pressure(Contributions::Total))?,
@@ -297,7 +297,7 @@ impl<U: EosUnit, E: EquationOfState> PhaseEquilibrium<U, E, 2> {
     }
 
     fn vle_init_stability(feed_state: &State<U, E>) -> EosResult<Self> {
-        let mut stable_states = feed_state.stability_analysis(VLEOptions::default())?;
+        let mut stable_states = feed_state.stability_analysis(SolverOptions::default())?;
         let state1 = stable_states.pop();
         let state2 = stable_states.pop();
         match (state1, state2) {
