@@ -2,10 +2,12 @@ use super::{Derivative::*, PartialDerivative, State};
 use crate::equation_of_state::{EntropyScaling, EquationOfState, MolarWeight};
 use crate::errors::EosResult;
 use crate::EosUnit;
-use ndarray::{Array1, Array2};
+use ndarray::{arr1, Array1, Array2};
 use num_dual::DualNum;
 use quantity::{QuantityArray, QuantityArray1, QuantityArray2, QuantityScalar};
+use std::iter::FromIterator;
 use std::ops::{Add, Sub};
+use std::rc::Rc;
 
 #[derive(Clone, Copy)]
 pub(crate) enum Evaluate {
@@ -17,13 +19,14 @@ pub(crate) enum Evaluate {
 
 /// Possible contributions that can be computed.
 #[derive(Clone, Copy)]
+#[cfg_attr(feature = "python", pyo3::pyclass)]
 pub enum Contributions {
     /// Only compute the ideal gas contribution
     IdealGas,
     /// Only compute the difference between the total and the ideal gas contribution
-    Residual,
+    ResidualNvt,
     /// Compute the differnce between the total and the ideal gas contribution for a (N,p,T) reference state
-    ResidualP,
+    ResidualNpt,
     /// Compute ideal gas and residual contributions
     Total,
 }
@@ -143,14 +146,14 @@ impl<U: EosUnit, E: EquationOfState> State<U, E> {
         match contributions {
             Contributions::IdealGas => f(self, Evaluate::IdealGas),
             Contributions::Total => f(self, Evaluate::Total),
-            Contributions::Residual => {
+            Contributions::ResidualNvt => {
                 if additive {
                     f(self, Evaluate::Residual)
                 } else {
                     f(self, Evaluate::Total) - f(self, Evaluate::IdealGas)
                 }
             }
-            Contributions::ResidualP => {
+            Contributions::ResidualNpt => {
                 let p = self.pressure_(Evaluate::Total);
                 let state_p = Self::new_nvt(
                     &self.eos,
@@ -292,11 +295,38 @@ impl<U: EosUnit, E: EquationOfState> State<U, E> {
 
     /// Logarithm of the fugacity coefficient: $\ln\varphi_i=\beta\mu_i^\mathrm{res}\left(T,p,\lbrace N_i\rbrace\right)$
     pub fn ln_phi(&self) -> Array1<f64> {
-        (self.chemical_potential(Contributions::ResidualP) / (U::gas_constant() * self.temperature))
+        (self.chemical_potential(Contributions::ResidualNpt)
+            / (U::gas_constant() * self.temperature))
             .into_value()
             .unwrap()
     }
 
+    /// Logarithm of the fugacity coefficient of all components treated as pure substance at mixture temperature and pressure.
+    pub fn ln_phi_pure(&self) -> EosResult<Array1<f64>> {
+        let pressure = self.pressure(Contributions::Total);
+        (0..self.eos.components())
+            .map(|i| {
+                let eos = Rc::new(self.eos.subset(&[i]));
+                let state = Self::new_npt(
+                    &eos,
+                    self.temperature,
+                    pressure,
+                    &(arr1(&[1.0]) * U::reference_moles()),
+                    crate::DensityInitialization::Liquid,
+                )?;
+                Ok(state.ln_phi()[0])
+            })
+            .collect()
+    }
+
+    /// Activity coefficient $\ln \gamma_i = \ln \varphi_i(T, p, \mathbf{N}) - \ln \varphi_i(T, p)$
+    pub fn ln_symmetric_activity_coefficient(&self) -> EosResult<Array1<f64>> {
+        match self.eos.components() {
+            1 => Ok(arr1(&[0.0])),
+            _ => Ok(self.ln_phi() - &self.ln_phi_pure()?),
+        }
+    }
+
     /// Partial derivative of the logarithm of the fugacity coefficient w.r.t. temperature: $\left(\frac{\partial\ln\varphi_i}{\partial T}\right)_{p,N_i}$
     pub fn dln_phi_dt(&self) -> QuantityArray1<U> {
         let func = |s: &Self, evaluate: Evaluate| {
@@ -304,18 +334,18 @@ impl<U: EosUnit, E: EquationOfState> State<U, E> {
                 - s.chemical_potential_(evaluate) / self.temperature)
                 / (U::gas_constant() * self.temperature)
         };
-        self.evaluate_property(func, Contributions::ResidualP, false)
+        self.evaluate_property(func, Contributions::ResidualNpt, false)
     }
 
     /// Partial derivative of the logarithm of the fugacity coefficient w.r.t. pressure: $\left(\frac{\partial\ln\varphi_i}{\partial p}\right)_{T,N_i}$
     pub fn dln_phi_dp(&self) -> QuantityArray1<U> {
-        self.molar_volume(Contributions::ResidualP) / (U::gas_constant() * self.temperature)
+        self.molar_volume(Contributions::ResidualNpt) / (U::gas_constant() * self.temperature)
     }
 
     /// Partial derivative of the logarithm of the fugacity coefficient w.r.t. moles: $\left(\frac{\partial\ln\varphi_i}{\partial N_j}\right)_{T,p,N_k}$
     pub fn dln_phi_dnj(&self) -> QuantityArray2<U> {
         let n = self.eos.components();
-        let dmu_dni = self.dmu_dni(Contributions::Residual);
+        let dmu_dni = self.dmu_dni(Contributions::ResidualNvt);
         let dp_dni = self.dp_dni(Contributions::Total);
         let dp_dv = self.dp_dv(Contributions::Total);
         let dp_dn_2 = QuantityArray::from_shape_fn((n, n), |(i, j)| dp_dni.get(i) * dp_dni.get(j));
@@ -615,13 +645,15 @@ impl<U: EosUnit, E: EquationOfState> State<U, E> {
 ///
 /// These properties are available for equations of state
 /// that implement the [EntropyScaling] trait.
-impl<U: EosUnit, E: EquationOfState + EntropyScaling<U, E>> State<U, E> {
+impl<U: EosUnit, E: EquationOfState + EntropyScaling<U>> State<U, E> {
     /// Return the viscosity via entropy scaling.
     pub fn viscosity(&self) -> EosResult<QuantityScalar<U>> {
         let s = self
-            .molar_entropy(Contributions::Residual)
+            .molar_entropy(Contributions::ResidualNvt)
             .to_reduced(U::reference_molar_entropy())?;
-        Ok(self.eos.viscosity_reference(self)?
+        Ok(self
+            .eos
+            .viscosity_reference(self.temperature, self.volume, &self.moles)?
             * self.eos.viscosity_correlation(s, &self.molefracs)?.exp())
     }
 
@@ -631,22 +663,25 @@ impl<U: EosUnit, E: EquationOfState + EntropyScaling<U, E>> State<U, E> {
     /// that is used for entropy scaling.
     pub fn ln_viscosity_reduced(&self) -> EosResult<f64> {
         let s = self
-            .molar_entropy(Contributions::Residual)
+            .molar_entropy(Contributions::ResidualNvt)
             .to_reduced(U::reference_molar_entropy())?;
         self.eos.viscosity_correlation(s, &self.molefracs)
     }
 
     /// Return the viscosity reference as used in entropy scaling.
     pub fn viscosity_reference(&self) -> EosResult<QuantityScalar<U>> {
-        self.eos.viscosity_reference(self)
+        self.eos
+            .viscosity_reference(self.temperature, self.volume, &self.moles)
     }
 
     /// Return the diffusion via entropy scaling.
     pub fn diffusion(&self) -> EosResult<QuantityScalar<U>> {
         let s = self
-            .molar_entropy(Contributions::Residual)
+            .molar_entropy(Contributions::ResidualNvt)
             .to_reduced(U::reference_molar_entropy())?;
-        Ok(self.eos.diffusion_reference(self)?
+        Ok(self
+            .eos
+            .diffusion_reference(self.temperature, self.volume, &self.moles)?
             * self.eos.diffusion_correlation(s, &self.molefracs)?.exp())
     }
 
@@ -656,22 +691,25 @@ impl<U: EosUnit, E: EquationOfState + EntropyScaling<U, E>> State<U, E> {
     /// that is used for entropy scaling.
     pub fn ln_diffusion_reduced(&self) -> EosResult<f64> {
         let s = self
-            .molar_entropy(Contributions::Residual)
+            .molar_entropy(Contributions::ResidualNvt)
             .to_reduced(U::reference_molar_entropy())?;
         self.eos.diffusion_correlation(s, &self.molefracs)
     }
 
     /// Return the diffusion reference as used in entropy scaling.
     pub fn diffusion_reference(&self) -> EosResult<QuantityScalar<U>> {
-        self.eos.diffusion_reference(self)
+        self.eos
+            .diffusion_reference(self.temperature, self.volume, &self.moles)
     }
 
     /// Return the thermal conductivity via entropy scaling.
     pub fn thermal_conductivity(&self) -> EosResult<QuantityScalar<U>> {
         let s = self
-            .molar_entropy(Contributions::Residual)
+            .molar_entropy(Contributions::ResidualNvt)
             .to_reduced(U::reference_molar_entropy())?;
-        Ok(self.eos.thermal_conductivity_reference(self)?
+        Ok(self
+            .eos
+            .thermal_conductivity_reference(self.temperature, self.volume, &self.moles)?
             * self
                 .eos
                 .thermal_conductivity_correlation(s, &self.molefracs)?
@@ -684,7 +722,7 @@ impl<U: EosUnit, E: EquationOfState + EntropyScaling<U, E>> State<U, E> {
     /// that is used for entropy scaling.
     pub fn ln_thermal_conductivity_reduced(&self) -> EosResult<f64> {
         let s = self
-            .molar_entropy(Contributions::Residual)
+            .molar_entropy(Contributions::ResidualNvt)
             .to_reduced(U::reference_molar_entropy())?;
         self.eos
             .thermal_conductivity_correlation(s, &self.molefracs)
@@ -692,6 +730,87 @@ impl<U: EosUnit, E: EquationOfState + EntropyScaling<U, E>> State<U, E> {
 
     /// Return the thermal conductivity reference as used in entropy scaling.
     pub fn thermal_conductivity_reference(&self) -> EosResult<QuantityScalar<U>> {
-        self.eos.thermal_conductivity_reference(self)
+        self.eos
+            .thermal_conductivity_reference(self.temperature, self.volume, &self.moles)
+    }
+}
+
+/// A list of states for a simple access to properties
+/// of multiple states.
+pub struct StateVec<'a, U, E> {
+    pub states: Vec<&'a State<U, E>>,
+}
+
+impl<'a, U, E> FromIterator<&'a State<U, E>> for StateVec<'a, U, E> {
+    fn from_iter<I: IntoIterator<Item = &'a State<U, E>>>(iter: I) -> Self {
+        Self {
+            states: iter.into_iter().collect(),
+        }
+    }
+}
+
+impl<'a, U: EosUnit, E: EquationOfState> StateVec<'a, U, E> {
+    pub fn temperature(&self) -> QuantityArray1<U> {
+        QuantityArray1::from_shape_fn(self.states.len(), |i| self.states[i].temperature)
+    }
+
+    pub fn pressure(&self) -> QuantityArray1<U> {
+        QuantityArray1::from_shape_fn(self.states.len(), |i| {
+            self.states[i].pressure(Contributions::Total)
+        })
+    }
+
+    pub fn compressibility(&self) -> Array1<f64> {
+        Array1::from_shape_fn(self.states.len(), |i| {
+            self.states[i].compressibility(Contributions::Total)
+        })
+    }
+
+    pub fn density(&self) -> QuantityArray1<U> {
+        QuantityArray1::from_shape_fn(self.states.len(), |i| self.states[i].density)
+    }
+
+    pub fn molefracs(&self) -> Array2<f64> {
+        Array2::from_shape_fn(
+            (self.states.len(), self.states[0].eos.components()),
+            |(i, j)| self.states[i].molefracs[j],
+        )
+    }
+
+    pub fn molar_enthalpy(&self) -> QuantityArray1<U> {
+        QuantityArray1::from_shape_fn(self.states.len(), |i| {
+            self.states[i].molar_enthalpy(Contributions::Total)
+        })
+    }
+
+    pub fn molar_entropy(&self) -> QuantityArray1<U> {
+        QuantityArray1::from_shape_fn(self.states.len(), |i| {
+            self.states[i].molar_entropy(Contributions::Total)
+        })
+    }
+}
+
+impl<'a, U: EosUnit, E: EquationOfState + MolarWeight<U>> StateVec<'a, U, E> {
+    pub fn mass_density(&self) -> QuantityArray1<U> {
+        QuantityArray1::from_shape_fn(self.states.len(), |i| self.states[i].mass_density())
+    }
+
+    pub fn massfracs(&self) -> Array2<f64> {
+        Array2::from_shape_fn(
+            (self.states.len(), self.states[0].eos.components()),
+            |(i, j)| self.states[i].massfracs()[j],
+        )
+    }
+
+    pub fn specific_enthalpy(&self) -> QuantityArray1<U> {
+        QuantityArray1::from_shape_fn(self.states.len(), |i| {
+            self.states[i].specific_enthalpy(Contributions::Total)
+        })
+    }
+
+    pub fn specific_entropy(&self) -> QuantityArray1<U> {
+        QuantityArray1::from_shape_fn(self.states.len(), |i| {
+            self.states[i].specific_entropy(Contributions::Total)
+        })
     }
 }
